@@ -13,7 +13,7 @@ from ..widgets.episode_row import EpisodeRow
 
 
 @Gtk.Template(resource_path=shared.PREFIX + '/ui/dialogs/edit_season.ui')
-class EditSeasonDialog(Adw.Window):
+class EditSeasonDialog(Adw.Dialog):
     """
     This class represents the window to edit a season.
 
@@ -46,7 +46,7 @@ class EditSeasonDialog(Adw.Window):
                  episodes: List[tuple] | None = None):
 
         super().__init__()
-        self.set_transient_for(parent)
+        self.parent = parent
 
         self._title = title
         self._poster_uri = poster_uri
@@ -77,6 +77,7 @@ class EditSeasonDialog(Adw.Window):
                                                 runtime=episode[2],
                                                 overview=episode[3],
                                                 still_uri=episode[4],
+                                                watched=episode[5],
                                                 editable=True)
                                      )
 
@@ -151,7 +152,8 @@ class EditSeasonDialog(Adw.Window):
                           episode_number: int,
                           runtime: int,
                           overview: str,
-                          still_uri: str) -> None:
+                          still_uri: str,
+                          watched: bool) -> None:
         """
         Callback for "edit-saved" signal.
         Appends the recieved data as a tuple in the episodes list and updates the ui.
@@ -168,7 +170,7 @@ class EditSeasonDialog(Adw.Window):
             None
         """
 
-        self._episodes.append((title, episode_number, runtime, overview, still_uri))
+        self._episodes.append((title, episode_number, runtime, overview, still_uri, watched))
         self.update_episodes_ui()
 
     def update_episodes_ui(self) -> None:
@@ -193,6 +195,7 @@ class EditSeasonDialog(Adw.Window):
                                                 runtime=episode[2],
                                                 overview=episode[3],
                                                 still_uri=episode[4],
+                                                watched=episode[5],
                                                 editable=True))
         self._enable_save_btn()
 
