@@ -9,7 +9,9 @@ from gettext import gettext as _
 from gettext import pgettext as C_
 from pathlib import Path
 
+import tmdbsimple
 from gi.repository import Adw, Gio, GLib, GObject, Gtk
+from requests import HTTPError
 
 from . import shared  # type: ignore
 from .background_queue import ActivityType, BackgroundActivity, BackgroundQueue
@@ -19,8 +21,8 @@ from .providers.tmdb_provider import TMDBProvider as tmdb
 
 
 @Gtk.Template(resource_path=shared.PREFIX + '/ui/preferences.ui')
-class PreferencesWindow(Adw.PreferencesWindow):
-    __gtype_name__ = 'PreferencesWindow'
+class PreferencesDialog(Adw.PreferencesDialog):
+    __gtype_name__ = 'PreferencesDialog'
 
     _download_group = Gtk.Template.Child()
     _language_comborow = Gtk.Template.Child()
@@ -29,8 +31,10 @@ class PreferencesWindow(Adw.PreferencesWindow):
     _offline_group = Gtk.Template.Child()
     _offline_switch = Gtk.Template.Child()
     _tmdb_group = Gtk.Template.Child()
+    _use_own_key_switch = Gtk.Template.Child()
+    _own_key_entryrow = Gtk.Template.Child()
     _housekeeping_group = Gtk.Template.Child()
-    _exit_cache_row = Gtk.Template.Child()
+    _exit_cache_switch = Gtk.Template.Child()
     _cache_row = Gtk.Template.Child()
     _data_row = Gtk.Template.Child()
 
@@ -50,13 +54,17 @@ class PreferencesWindow(Adw.PreferencesWindow):
 
         shared.schema.bind('offline-mode', self._offline_switch,
                            'active', Gio.SettingsBindFlags.DEFAULT)
-        shared.schema.bind('exit-remove-cache', self._exit_cache_row,
+        shared.schema.bind('use-own-tmdb-key', self._use_own_key_switch,
+                           'active', Gio.SettingsBindFlags.DEFAULT)
+        shared.schema.bind('exit-remove-cache', self._exit_cache_switch,
                            'active', Gio.SettingsBindFlags.DEFAULT)
 
         self._offline_switch.connect('notify::active', lambda pspec, user_data: logging.debug(
             f'Toggled offline mode: {self._offline_switch.get_active()}'))
-        self._exit_cache_row.connect('notify::active', lambda pspec, user_data: logging.debug(
-            f'Toggled clear cache on exit: {self._offline_switch.get_active()}'))
+        self._use_own_key_switch.connect(
+            'notify::active', self._on_use_own_key_switch_activated)
+        self._exit_cache_switch.connect('notify::active', lambda pspec, user_data: logging.debug(
+            f'Toggled clear cache on exit: {self._exit_cache_switch.get_active()}'))
 
     @Gtk.Template.Callback('_on_map')
     def _on_map(self, user_data: object | None) -> None:
@@ -85,7 +93,10 @@ class PreferencesWindow(Adw.PreferencesWindow):
             case 'month':
                 self._update_freq_comborow.set_selected(3)
 
-        # Update check
+        self._own_key_entryrow.set_text(
+            shared.schema.get_string('own-tmdb-key'))
+
+        # Update occupied space
         self._update_occupied_space()
 
     def _setup_languages(self):
@@ -223,22 +234,27 @@ class PreferencesWindow(Adw.PreferencesWindow):
             languages = tmdb.get_languages()
             for lang in languages:
                 local.add_language(LanguageModel(lang))
+                
+            local.update_movies_table()
+            local.update_series_table()
 
             shared.schema.set_boolean('first-run', False)
             shared.schema.set_boolean('offline-mode', False)
             shared.schema.set_boolean('onboard-complete', True)
+            shared.schema.set_boolean('db-needs-update', False)
 
             self._setup_languages()
+            
             Gio.NetworkMonitor.get_default().connect(
                 'network-changed', self._on_network_changed)
         else:
             logging.error('No network, aborting first setup completion')
-            dialog = Adw.MessageDialog.new(self,
-                                           C_('message dialog heading',
-                                              'No Network'),
-                                           C_('message dialog body', 'Connect to the Internet to complete the setup.'))
-            dialog.add_response('ok', C_('message dialog action', 'OK'))
-            dialog.present()
+            dialog = Adw.AlertDialog.new(
+                heading=C_('message dialog heading', 'No Network'),
+                body=C_('message dialog body', 'Connect to the Internet to complete the setup.')
+            )
+            dialog.add_response('ok', C_('alert dialog action', '_OK'))
+            dialog.choose(self, None, None, None)
 
     def _on_network_changed(self, network_monitor: Gio.NetworkMonitor, network_available: bool) -> None:
         """
@@ -273,8 +289,7 @@ class PreferencesWindow(Adw.PreferencesWindow):
         builder = Gtk.Builder.new_from_resource(
             shared.PREFIX + '/ui/dialogs/message_dialogs.ui')
         _clear_cache_dialog = builder.get_object('_clear_cache_dialog')
-        _clear_cache_dialog.set_transient_for(self)
-        _clear_cache_dialog.choose(
+        _clear_cache_dialog.choose(self,
             None, self._on_cache_message_dialog_choose, None)
 
     def _on_cache_message_dialog_choose(self,
@@ -294,7 +309,7 @@ class PreferencesWindow(Adw.PreferencesWindow):
             None
         """
 
-        result = Adw.MessageDialog.choose_finish(source, result)
+        result = Adw.AlertDialog.choose_finish(source, result)
         if result == 'cache_cancel':
             logging.debug('Cache clear dialog: cancel, aborting')
             return
@@ -360,8 +375,7 @@ class PreferencesWindow(Adw.PreferencesWindow):
         _series_row.set_subtitle(_('{number} Titles').format(
             number=len(local.get_all_series())))
 
-        _clear_data_dialog.set_transient_for(self)
-        _clear_data_dialog.choose(
+        _clear_data_dialog.choose(self,
             None, self._on_data_message_dialog_choose, None)
 
     def _on_data_message_dialog_choose(self,
@@ -374,7 +388,7 @@ class PreferencesWindow(Adw.PreferencesWindow):
 
         Args:
             source (Gtk.Widget): object that started the async operation
-            result (Gio.AsyncResult): a Gio.AsyncResultresult = Adw.MessageDialog.choose_finish(source, result)
+            result (Gio.AsyncResult): a Gio.AsyncResultresult = Adw.AlertDialog.choose_finish(source, result)
         if result == 'data_cancel':
             return
 
@@ -384,7 +398,7 @@ class PreferencesWindow(Adw.PreferencesWindow):
             None
         """
 
-        result = Adw.MessageDialog.choose_finish(source, result)
+        result = Adw.AlertDialog.choose_finish(source, result)
         if result == 'data_cancel':
             logging.debug('Data clear dialog: cancel, aborting')
             return
@@ -434,7 +448,7 @@ class PreferencesWindow(Adw.PreferencesWindow):
         """Callback to complete async activity"""
 
         self._update_occupied_space()
-        self.get_transient_for().activate_action('win.refresh', None)
+        self.get_parent().activate_action('win.refresh', None)
         activity.end()
 
     def _clear_series(self, activity: BackgroundActivity) -> None:
@@ -488,3 +502,60 @@ class PreferencesWindow(Adw.PreferencesWindow):
             _('{space:.2f} MB occupied').format(space=cache_space))
         self._data_row.set_subtitle(
             _('{space:.2f} MB occupied').format(space=data_space))
+
+    @Gtk.Template.Callback('_on_own_key_changed')
+    def _on_own_key_changed(self, user_data: object | None) -> None:
+        """
+        Callback for 'changed' signal.
+        Removes all CSS classes.
+
+        Args:
+            user_data (object or None): additional data passed to the callback
+
+        Returns:
+            None
+        """
+
+        self._own_key_entryrow.remove_css_class('error')
+        self._own_key_entryrow.remove_css_class('success')
+
+    @Gtk.Template.Callback('_on_check_own_key_button_clicked')
+    def _on_check_own_key_button_clicked(self, user_data: object | None) -> None:
+        """
+        Callback for 'clicked' signal.
+        Temporarelly changes the API key in use with the one provided and checks if it is valid.
+
+        Args:
+            user_data (object or None): additional data passed to the callback
+
+        Returns:
+            None
+        """
+
+        tmdb.set_key(self._own_key_entryrow.get_text())
+        try:
+            tmdb.get_movie(575264)
+            shared.schema.set_string(
+                'own-tmdb-key', self._own_key_entryrow.get_text())
+            self._own_key_entryrow.add_css_class('success')
+        except (tmdbsimple.base.APIKeyError, HTTPError):
+            self._own_key_entryrow.add_css_class('error')
+            tmdb.set_key(tmdb.get_builtin_key())
+
+    def _on_use_own_key_switch_activated(self, pspec, user_data: object | None) -> None:
+        """
+        Callback for 'notify::active' signal.
+        Sets the API key is use based on the state of the property.
+
+        Args:
+            pspec (GObject.ParamSpec): pspec of the changed property
+            user_data (object or None): additional data passed to the callback
+
+        Returns:
+            None
+        """
+
+        if self._use_own_key_switch.get_active():
+            tmdb.set_key(self._own_key_entryrow.get_text())
+        else:
+            tmdb.set_key(tmdb.get_builtin_key())
